<?php

namespace core;

class CheckLogin
{
    private static $is_login_query = null;

    private static $lifetime = 604800;

    public function __construct()
    {
        $admin = include ROOT . '/config/admin.php';
        $this->lifetime = $admin['lifetime'];
    }

    public static function auth()
    {
        if (!is_null(\g::settings('is_russian')) && (int)\g::settings('is_russian') === 1) {
            include ROOT . '/miwix-admin/vendors/SxGeo.php';
            $SxGeo = new \SxGeo(ROOT . '/miwix-admin/vendors/SxGeo.dat', SXGEO_BATCH | SXGEO_MEMORY);
            $ip = $_SERVER['REMOTE_ADDR'];
            $country = $SxGeo->getCountry($ip);
            if ($country !== 'RU') {
                return false;
            }


        }

        $data = \g::request()->post()->get();
        if (empty($data)) {
            return false;
        }
        if ($data['username'] == '' || $data['password'] == '') {
            return false;
        }
        if (!preg_match("|[A-zА-яЁ ]+|", $data['username'])) {
            return false;
        }

        $password = password_hash($data['password'], PASSWORD_DEFAULT);
        $query = \g::db()->query('SELECT `id`, `password`, `ban_time`, `attempts`, `login` FROM `accesses` WHERE `login` = ?s AND `active` = 1 LIMIT 1', $data['username']);

        if ($query->num_rows) {
            $admin = $query->fetch_assoc();
        } else {
            return false;
        }

        if ($admin['ban_time'] > time())
        {
            return false;
        }

        if (password_verify($data['password'], $admin['password']) == 1) {
            $hash_password = hash('sha512', hash('sha512', $data['username']) . hash('sha512', $password) . session_id());
            $_SESSION['login'] = $data['username'];
            $_SESSION['hash_password'] = $hash_password;
            self::setTimeAuth();

            $ip = empty($ip = ip2long($_SERVER['REMOTE_ADDR'])) ? 0 : $ip;
            \g::db()->query('UPDATE `accesses` SET `hash` = ?s, `ip` = ?i, `ban_time` = ?i, `attempts` = 0 WHERE `login` = ?s', $hash_password, $ip, time() - 10, $data['username']);
            return true;
        } else {
            if ($admin['attempts'] > 4)
            {
                \g::db()->query('UPDATE `accesses` SET `attempts` = 0, `ban_time` = ?i WHERE `login` = ?s', time() + (60 * 15), $data['username']);
            } else {
                \g::db()->query('UPDATE `accesses` SET `attempts` = `attempts` + 1 WHERE `login` = ?s', $data['username']);
            }
        }
    }

    public static function is_secret_admin()
    {
		if (\g::settings('secret_admin') != "") {
            $secret = \g::request()->get()->get('secret_admin');
			if ($secret && !empty(trim($secret))) {
                if (\g::settings('secret_admin') === $secret) {
                    return true;
                }
            }
		}
        \core\Errors::redirect404();
    }

    public static function is_login()
    {
        if (!isset($_SESSION['hash_password']) || empty(trim($_SESSION['hash_password'])) || !isset($_SESSION['lifetime'])) {
            return false;
        } else {
            if ($_SESSION['lifetime'] < time()) {
                return false;
            }

            $ip = empty($ip = ip2long($_SERVER['REMOTE_ADDR'])) ? 0 : $ip;

            if (is_null(self::$is_login_query)) {
                $query = \g::db()->query("SELECT `id` FROM `accesses` WHERE `login` = ?s AND `hash` = ?s AND `ip` = ?i LIMIT 1", $_SESSION['login'], $_SESSION['hash_password'], $ip);
                self::$is_login_query = $query->num_rows;
            }
            if (self::$is_login_query) {
                self::setTimeAuth();
                return true;
            }
        }
        return false;
    }

    public static function is_access($controller)
    {
        $ip = empty($ip = ip2long($_SERVER['REMOTE_ADDR'])) ? 0 : $ip;
        $query = \g::db()->query("SELECT `privilege` FROM `accesses` WHERE `login`=?s AND `hash`=?s AND `ip`=?i", $_SESSION['login'], $_SESSION['hash_password'], $ip);
        if ($query->num_rows > 0) {
            $privilege = explode('|', $query->fetch_row()[0]);
            if (array_search($controller, $privilege) === false) {
                \g::response()->data()->set(['res' => false, 'message' => 'Доступ запрещен']);
                \core\response\Json::run();
            }
        } else {
            \g::response()->data()->set(['res' => false, 'message' => 'Доступ запрещен']);
            \core\response\Json::run();
        }
    }


    public static function is_access_method($controller)
    {
        $ip = empty($ip = ip2long($_SERVER['REMOTE_ADDR'])) ? 0 : $ip;
        $query = \g::db()->query("SELECT `privilege` FROM `accesses` WHERE `login`=?s AND `hash`=?s AND `ip`=?i", $_SESSION['login'], $_SESSION['hash_password'], $ip);
        if ($query->num_rows > 0) {
            $privilege = explode('|', $query->fetch_row()[0]);
            foreach ($privilege as $item) {
                if (mb_strtolower($item) == mb_strtolower($controller))
                    return true;
            }
            if (self::is_access_demo($controller)) {
                return true;
            }
        }
        return false;
    }

    public static function is_access_demo($controller = '')
    {
        $ip = empty($ip = ip2long($_SERVER['REMOTE_ADDR'])) ? 0 : $ip;
        $query = \g::db()->query("SELECT `privilege` FROM `accesses` WHERE `login`=?s AND `hash`=?s AND `ip`=?i AND `privilege` = ?s", $_SESSION['login'], $_SESSION['hash_password'], $ip, 'Demo');
        if ($query->num_rows > 0 && !in_array($controller, array('adminer', 'helper'))) {
            if (!empty($query->fetch_row()[0])) {
                return true;
            }
        }
        return false;
    }

    public static function exitAdmin()
    {
        unset($_SESSION['hash_password'], $_SESSION['login']);
        $_SESSION['checkExitLogin'] = true;
        header('Location: /');
    }
	
    public function checkFilesGreen()
    {
        if (self::module('files-root'))
            return true;
        else
            return false;
    }

    private static function setTimeAuth()
    {
        $_SESSION['lifetime'] = time() + self::$lifetime;
    }

    private static function module($name = '')
    {
        $name = trim($name);
        if (!$name)
            return;

        $name = '"%' . $name . '%"';

        if (!isset($_SESSION['hash_password']) || trim($_SESSION['hash_password']) == '') {
            return;
        } else {
            $login = $_SESSION['login'];
            $hash_password = $_SESSION['hash_password'];
            $ip = empty($ip = ip2long($_SERVER['REMOTE_ADDR'])) ? 0 : $ip;
            $count = \g::db()->getRow('SELECT count(*) AS `count` FROM `accesses` WHERE `login`=?s AND `hash`=?s AND `ip`=?i AND `privilege` LIKE ?p LIMIT 1', $login, $hash_password, $ip, $name);

            if ($count['count'] == 1)
                return true;
        }
        return false;
    }

    private static function userBlockListCount()
    {
        $dir = ROOT . '/block_list/' . '/' . date('d.m.Y') . '/';
        $file_log = $dir . $_SERVER['REMOTE_ADDR'] . '.log';

        if (is_file($file_log))
            $count = file_get_contents($file_log);
        else {
            if (!is_dir($dir))
                mkdir($dir, 0700, true);
            file_put_contents($file_log, 0);
            $count = 0;
        }

        if ($count > 5)
            self::blockUser();
        else
            $count++;
        file_put_contents($file_log, $count);
        return false;
    }

    private static function blockUser()
    {
        if (isset($_SESSION['login'])) {
            $query = \g::db()->query("SELECT `id` FROM `accesses` WHERE `login` = ?s", $_SESSION['login']);
            if ($query->num_rows > 0) {
                $id = $query->fetch_row()[0];
                \g::db()->query('UPDATE `accesses` SET `hash` = ?s, `ip` = ?s, `active` = 0 WHERE `id` = ?i', 0, $_SERVER['REMOTE_ADDR'], $id);
            }
        }
        if ($query->num_rows === 0)
            \g::db()->query("INSERT INTO `block_list` (`ip`, `time`) VALUES (?s, ?i)", $_SERVER['REMOTE_ADDR'], time());
        return true;
    }

    private static function isBlockUser()
    {
        $query = \g::db()->query("SELECT `id` FROM `block_list` WHERE `ip` = ?s", $_SERVER['REMOTE_ADDR']);
        if ($query->num_rows > 0)
            return true;
        return false;
    }
}
