<?php

namespace core;

class ModuleLoader
{

    private $modules = null;
    private $folder;
    private $currentVersion = 1.0;

    private $md5_dirs_list = [];

    static private $moduleCheck = [];
    static private $infoModules = null;


    public function __construct($folder)
    {
        $this->folder = $folder;
    }

    public function load()
    {
        if (!is_null($this->modules))
            return $this->modules;
        $ml_time_start = microtime(true);
        if ($handle = opendir(__DIR__ . '/../../' . $this->folder)) {
            while ($entry = readdir($handle)) {
                if (strpos($entry, '.') !== false) {
                    continue;
                }
                /*
                 * Ищем класс модуля, если он инвалидный - модуль такой не загружаем
                 */
                if (!file_exists(ROOT . '/' . $this->folder . '/' . $entry . '/Module.php')) {
                    continue;
                }
                $moduleInfo = include ROOT . '/' . $this->folder . '/' . $entry . '/Module.php';

                $this->modules[] = $entry;
            }
            closedir($handle);
        }
        $this->modules = $this->prioritySort($this->modules);
    }

    public function prioritySort($array)
    {
        $dump = [];
        foreach ($array as $row) {
            $moduleInfo = include ROOT . '/' . $this->folder . '/' . $row . '/Module.php';
            if (!isset($moduleInfo['priority'])) {
                $dump[] = array_merge(array("name" => $row), array("priority" => 0));
            } else {
                $dump[] = array_merge(array("name" => $row), array("priority" => $moduleInfo['priority']));
            }
        }
        uasort($dump, array($this, 'sortStage'));
        return $dump;
    }

    public function sortStage($a, $b)
    {
        if ($a['priority'] === $b['priority']) {
            return 0;
        }
        if ($a['priority'] < $b['priority']) {
            return 1;
        }
        return -1;
    }

    public function MD5_DIR($dir)
    {
        if (array_key_exists($dir, $this->md5_dirs_list))
            return $this->md5_dirs_list[$dir];

        if (!is_dir($dir)) {
            return false;
        }

        $filemd5s = array();
        $d = dir($dir);

        while (false !== ($entry = $d->read())) {
            if ($entry != '.' && $entry != '..' && $entry != '.DS_store' && $entry != 'Config.php') {
                if (is_dir($dir . '/' . $entry)) {
                    $filemd5s[] = $this->MD5_DIR($dir . '/' . $entry);
                } else {
                    $filemd5s[] = md5_file($dir . '/' . $entry);
                }
            }
        }
        $d->close();
        usort($filemd5s, function ($a, $b) {
            if ($a === $b) return 0;
            return $a > $b ? -1 : 1;
        });
        $result = md5(implode('', $filemd5s));

        $this->md5_dirs_list[$dir] = $result;

        return $result;
    }

    public function getModuleInfo($name)
    {
        foreach ($this->infoModules() as $module) {
            if (mb_strtolower($module['name']) == mb_strtolower($name))
                return $module;
        }

        return null;
    }

    public function getModulesLoaded()
    {
        return $this->modules;
    }

    /*
     * Получаем информацию о всех модулях, версии, названия, автора
     */
    public function infoModules()
    {
        if (!is_null(self::$infoModules))
            return self::$infoModules;

        $info = array();
        foreach ($this->modules as $module) {
            $moduleInfo = include ROOT . '/' . $this->folder . '/' . $module['name'] . '/Module.php';
            $info[] = array_merge($moduleInfo, array("hash" => $this->MD5_DIR(ROOT . '/' . $this->folder . '/' . $module['name']), "entry" => $module['name']));
        }
        self::$infoModules = $info;
        return $info;
    }

    // Проверим существует ли такой модуль по его названию
    public function checkModule($nameMod)
    {
        if (array_key_exists($nameMod, self::$moduleCheck))
            return self::$moduleCheck[$nameMod];

        foreach ($this->infoModules() as $module) {
            if (mb_strtolower($module['name']) == mb_strtolower($nameMod)) {
                self::$moduleCheck[$nameMod] = true;
                return true;
            }
        }
        self::$moduleCheck[$nameMod] = false;
        return false;
    }

    /*
        Выполнение метода во всех плагинах
    */
    public function fire($event, $currentModule = false)
    {
        foreach ($this->modules as $module) {
            if ($currentModule) {
                if (mb_strtolower($module['name']) != mb_strtolower(\g::stack()->get('route')['modules'])) {
                    continue;
                }
            }
            if (!file_exists(ROOT . '/' . $this->folder . '/' . $module['name'] . '/Module.php')) {
                continue;
            }
            $moduleInfo = include ROOT . '/' . $this->folder . '/' . $module['name'] . '/Module.php';
            if (isset($moduleInfo['target']) && $moduleInfo['target'] > $this->currentVersion) {
                continue;
            }
			if (isset($moduleInfo['handlers'])) {
				foreach ($moduleInfo['handlers'] as $handler) {
					$handlerPath = ROOT . '/' . $this->folder . '/' . $module['name'] . '/' . str_replace('.', '/', $handler) . '.php';
					if (file_exists($handlerPath)) {
						require_once $handlerPath;
						$handlerClass = explode('.', $handler)[count(explode('.', $handler)) - 1];
						$callable = new $handlerClass();
						$method = 'on' . $event->getEventName();
						if (method_exists($callable, $method)) {
							$callable->$method($event);
							if ($method === 'onRouteEvent') {
								if ($event->isRenderStopped()) {
									return true;
								}
							}
						}
					}
				}
			}
        }
    }

}
