<?php
namespace core;

abstract class Request
{
	protected $_stack = [];
	protected $errors = [];

	abstract public function sendErrors();

	public function get($key = null) {
		if (is_null($key)) {
			return $this->_stack;
		}
		if (isset($this->_stack[$key])) {
			return $this->_stack[$key];
		}
		$this->errors[] = [
			'name' => 'Ошибка получение данных',
			'message' => 'Ошибка получение данных по ключу '.$key.'. Данные отсутствуют'
		];
		return false;
	}

	public function set($key, $value): bool {
		if ($this->isCheckExistence($key)) {
			$this->errors[] = [
				'name' => 'Ошибка добавления данных',
				'message' => 'Ошибка добавления данных по ключу '.$key.'. Данные отсутствуют'
			];
			return false;
		}
		$this->_stack[$key] = $value;
		return true;
	}

	public function update($key, $value): bool {
		if ($this->isCheckExistence($key)) {
			$this->_stack[$key] = $value;
			return true;
		}
		$this->errors[] = [
			'name' => 'Ошибка обновления данных',
			'message' => 'Ошибка обновления данных по ключу '.$key.'. Данные отсутствуют'
		];
		return false;
	}

	public function remove($key): bool {
		if ($this->isCheckExistence($key)) {
			unset($this->_stack[$key]);
			return true;
		}
		$this->errors[] = [
			'name' => 'Ошибка удаления данных',
			'message' => 'Ошибка удаления данных по ключу '.$key.'. Данные отсутствуют'
		];
		return false;
	}

	public function clear() {
		$this->_stack = [];
		return true;
	}

	public function isCheckExistence($key): bool {
		return isset($this->_stack[$key]);
	}

	public function isErrors(): bool {
		return empty($this->errors) ? false : true;
	}
}
