<?php
namespace core;

abstract class Response
{
	protected $errors = [];
	protected $_stack;

	abstract public function sendErrors();

	public function get($key = null) {
		if (is_null($key)) {
			return $this->_stack;
		}
		if ($this->isCheckExistence($key)) {
			return $this->_stack[$key];
		}
		$this->errors[] = [
			'name' => 'Ошибка получение данных',
			'message' => 'Ошибка получения данных по ключу. Переменная с ключем '.$key.' не существует'
		];
		return false;
	}

	public function set($key, $data = null): bool {
		if (is_null($key) || empty($key)) {
			$this->errors[] = [
				'name' => 'Ошибка добавления',
				'message' => 'Ошибка добавления значение по ключу. Ключ не может быть пустым'
			];
			return false;
		}

		if (is_array($key) && is_null($data)) {
			foreach ($key as $k => $v) {
				$this->_stack[$k] = $v;
			}
			return true;
		}

		if ($this->isCheckExistence($key)) {
			$this->errors[] = [
				'name' => 'Ошибка добавления',
				'message' => 'Ошибка добавления значение по ключу. Переменная с ключем '.$key.' уже существует'
			];
			return false;
		}
		$this->_stack[$key] = $data;
		return true;
	}

	public function update($key, $data): bool {
		if (is_array($key) && is_null($data)) {
			foreach ($key as $k => $v) {
				$this->_stack[$k] = $v;
			}
			return true;
		}
		if ($this->isCheckExistence($key)) {
			$this->_stack[$key] = $data;
			return true;
		}
		$this->errors[] = [
			'name' => 'Ошибка обновления',
			'message' => 'Ошибка обновления значение по ключу '.$key.'. Переменная с ключем '.$key.' ещё не создана'
		];
		return false;
	}

	public function remove($key): bool {
		if ($this->isCheckExistence($key)) {
			unset($this->_stack[$key]);
			return true;
		}
		$this->errors[] = [
			'name' => 'Ошибка удаления',
			'message' => 'Ошибка удаления значение по ключу '.$key.'. Переменная с ключем '.$key.' ещё не создана'
		];
		return false;
	}

	public function clear(): bool {
		$this->_stack = [];
		return true;
	}

	public function isCheckExistence($key): bool {
		return isset($this->_stack[$key]);
	}

	public function isErrors(): bool {
		return empty($this->errors) ? false : true;
	}
}
