<?php
namespace core\request;

class Server
{
	private $_stack = [];
	private $_url;

	public function __construct() {
		$this->_stack = $_SERVER;
	}

	public function get($key = null) {
		if (is_null($key)) {
			return $this->_stack;
		} else if (is_array($key)) {
			$result = [];
			foreach ($key as $v) {
				if (isset($this->_stack[$v])) {
					$result[$v] = $this->_stack[$v];
				}
			}
			if (empty($result)) {
				return false;
			}
			return $result;
		} else {
			return isset($this->_stack[$key]) ? $this->_stack[$key] : false;
		}
	}

	public function getBrowser() {
		$user_agent = $this->_stack["HTTP_USER_AGENT"];
		if (strpos($user_agent, "Firefox") !== false) $browser = "Firefox";
		elseif (strpos($user_agent, "Opera") !== false) $browser = "Opera";
		elseif (strpos($user_agent, "Chrome") !== false) $browser = "Chrome";
		elseif (strpos($user_agent, "MSIE") !== false) $browser = "Internet Explorer";
		elseif (strpos($user_agent, "Safari") !== false) $browser = "Safari";
		else $browser = "Неизвестный";
		return $browser;
	}

	public function getDomains() {
		return sprintf(
			"%s://%s",
			isset($this->_stack['HTTPS']) && $this->_stack['HTTPS'] != 'off' ? 'https' : 'http',
			$this->_stack['SERVER_NAME']
		);
	}

	public function getIp() {
		if (!empty($this->_stack['HTTP_CLIENT_IP'])) {
			return $this->_stack['HTTP_CLIENT_IP'];
		} elseif (!empty($this->_stack['HTTP_X_FORWARDED_FOR'])) {
			return $this->_stack['HTTP_X_FORWARDED_FOR'];
		} else {
			return $this->_stack['REMOTE_ADDR'];
		}
	}

    public function updateUrl($url){
        $this->_url = $url;
    }

	public function getUrl() {
		if (empty($this->_url)) {
			$url = trim(urldecode(parse_url($this->_stack['REQUEST_URI'], PHP_URL_PATH)), '/');
			$this->_url = empty($url) ? 'index' : $url;
		}
		return $this->_url;
	}
}
