<?php

namespace globals;

class Base
{
    private $exceptions_urls = [];
    private $siteMapTable = [];

    public function findEntry($name)
    {
        $info = \g::ml()->infoModules();
        foreach ($info as $row) {
            if ($row['name'] === $name) {
                return $row['entry'];
            }
        }
        return false;
    }

    public function __construct()
    {
		
        $this->exceptions_urls = ['cart', '404', 'pages', 'users', 'signup', 'login', 'checkout'];
        $this->siteMapTable = [
			'pages' => [
				'prefix_url' => false,
				'time_create' => 'time',
				'time_update' => 'edit_time'
			], 
			'catalog_products' => [
				'prefix_url' => 'catalog/product',
				'time_create' => 'time_creation',
				'time_update' => 'last_modified'
			], 
			'catalog_categories' => [
				'prefix_url' => 'catalog/category',
				'time_create' => 'time_creation',
				'time_update' => 'last_modified'
			],
		];
    }

    public function generateSiteMap($sitemap = 'sitemap.xml'): bool
    {

        set_time_limit(0);
        $oGenerateSiteMapEvent = new \core\event\events\GenerateSiteMapEvent();
        \g::ml()->fire($oGenerateSiteMapEvent);
        $xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
        $xml .= "<urlset xmlns=\"http://www.sitemaps.org/schemas/sitemap/0.9\">\n";
        $host = \g::request()->server()->getDomains();
        if (method_exists(\g::Multilingual(), 'getLangList')) {
            \g::Multilingual()->createTableLangCatalog();
            \g::Multilingual()->createTableLangPages();
                $this->siteMapTable['catalog_products_lang'] = [
                    'prefix_url' => 'catalog/product',
                    'time_create' => 'time_creation',
                    'time_update' => 'last_modified'
                ];
                $this->siteMapTable['catalog_categories_lang'] = [
                    'prefix_url' => 'catalog/category',
                    'time_create' => 'time_creation',
                    'time_update' => 'last_modified'
                ];
                $this->siteMapTable['pages_lang'] = [
                    'prefix_url' => false,
                    'time_create' => 'time',
                    'time_update' => 'edit_time'
                ];
        }

        foreach ($this->siteMapTable as $table => $param) {
            if ($table == 'catalog_categories' || $table == 'catalog_products') {
                $entry = $this->findEntry('Catalog');
                if (!$entry) {
                    continue;
                }
            }
            $time_create = $param['time_create'];
            $time_update = $param['time_update'];
            $prefix_url = false;

            switch ($table) {
                case 'pages':
                    $query = \g::db()->query("
                            SELECT `p`.`id`, `p`.`url`, `p`.`prefix_url`, `p`.?n, `p`.?n
							FROM `pages` as `p` 
                            LEFT JOIN `pages` as `c` ON `p`.`parent_url` = `c`.`url`
							WHERE `p`.`active` = 1 AND `p`.`site_map` = 1 AND COALESCE(c.active, 1) = 1 GROUP BY `p`.`id`", $time_create, $time_update);
                    break;
                case 'pages_lang':
                    $query = \g::db()->query("
                            SELECT p.`id`, p.`url`, p.`prefix_url`, l.`lang`, `p`.?n, `p`.?n FROM `pages_lang` as l 
                            LEFT JOIN `pages` as p ON l.PARENT_ID = p.id  WHERE p.`active` = 1 AND p.`site_map` = 1", $time_create, $time_update);
                    break;
                case 'catalog_categories_lang':
                    $query = \g::db()->query("
                            SELECT p.`id`, p.`url`, `p`.?n, `p`.?n, l.`lang` FROM `catalog_categories_lang` as l 
                            LEFT JOIN `catalog_categories` as p ON l.PARENT_ID = p.id  WHERE p.`active` = 1 AND p.`site_map` = 1", $time_create, $time_update);
                    break;
                case 'catalog_products_lang':
                    $query = \g::db()->query("
                            SELECT p.`id`, p.`url`, `p`.?n, `p`.?n, l.`lang` FROM `catalog_products_lang` as l 
                            LEFT JOIN `catalog_products` as p ON l.PARENT_ID = p.id  WHERE p.`active` = 1 AND p.`site_map` = 1", $time_create, $time_update);
                    break;
                case 'catalog_products':
                    $query = \g::db()->query("
                            SELECT `c`.`id`, `c`.`url`, `c`.?n, `c`.?n FROM `catalog_products` as `c` 
                            INNER JOIN `catalog_relations` as r ON c.id = r.cid AND r.type=1
                            INNER JOIN `catalog_categories` as p ON r.pid = p.id AND p.active=1
							WHERE c.`active` = 1 AND c.`site_map` = 1 GROUP BY c.id", $time_create, $time_update);
                    break;
				case 'catalog_categories':
                    $query = \g::db()->query("
                            SELECT `c`.`id`, `c`.`url`, `c`.?n, `c`.?n FROM `catalog_relations` as `r` 
                            INNER JOIN `catalog_categories` as `c` ON `r`.`cid` = `c`.`id` AND `c`.`active` = 1 AND `c`.`site_map` = 1
							WHERE `r`.`type`=2 GROUP BY `c`.`id`", $time_create, $time_update);
                    break;
                default:
                    $query = \g::db()->query("SELECT `id`, `url`, ?n, ?n FROM `$table` WHERE `active` = 1", $time_create, $time_update);
                    break;
            }
            if ($param['prefix_url'] !== false) {
                $prefix_url = $param['prefix_url'];
            }
            if ($query->num_rows) {
                while ($row = $query->fetch_assoc()) {
                    if (!in_array($row['url'], $this->exceptions_urls)) {
						$new_time = ($row[$time_update] > 0) ? $row[$time_update] : $row[$time_create];
                        if (array_key_exists('prefix_url', $row)) {
                            $prefix_url = $row['prefix_url'];
                        }
                        if (isset($row['lang']))
                            $row['url'] = $row['lang'] . "__" . $row['url'];
                        $xml .= "<url>\n";
                        $xml .= "<loc>" . $host . \f::generateUrl($row['url'], $prefix_url, 'pages') . "</loc>\n";
                        $xml .= "<lastmod>" . date('c', $new_time) . "</lastmod>\n";
                        $xml .= "</url>\n";
                    }
                }
            }
        }
        $xml .= '</urlset>';
        $fp = file_put_contents(ROOT . '/' . $sitemap, $xml);

        if (file_exists($fp) && is_file($fp))
            return true;
        return false;
    }

    public function rssTape($rss = 'rss.xml')
    {
        set_time_limit(0);
        $query = \g::db()->query("
			SELECT * 
			FROM `pages` 
			WHERE 
				`parent_url` = ?s OR 
				`parent_url` = ?s OR 
				`parent_url` = ?s OR 
				`parent_url` = ?s
			ORDER BY `time` DESC",
            'news', 'stati', 'foto', 'telestudiya'
        );
        if ($query->num_rows > 0) {
            $xml = '<?xml version="1.0" encoding="UTF-8"?>
<rss  version="2.0">
	<channel>
		<title>RSS | ' . \g::settings('site_name') . '</title>
		<link>' . $this->getBaseurl() . '/</link>
		<language>ru</language>';
            while ($row = $query->fetch_assoc()) {
                $xml .= '
		<item>';
                $xml .= '
			<title>' . $this->html($row['title']) . '</title>
			<link>' . $this->getBaseurl() . '/' . $row['url'] . '/</link>
			<pubDate>' . date('r', $row['time']) . '</pubDate>
			<description>' . $this->html($row['description']) . '</description>';
                if ($row['parent_url'] == 'foto') {
                    $matches = [];
                    preg_match_all("|\[g\[_(.*)_\]g\]|U", $row['content'], $matches, PREG_PATTERN_ORDER);
                    foreach ($matches[1] as $key => $name) {
                        $q = \g::db()->query("
						SELECT `gm`.`name` AS `name`, `g`.*
						FROM `gallery_main` AS `gm`
						INNER JOIN `gallery` AS `g` ON `g`.`gid` = `gm`.`id`
						WHERE `gm`.`name` = ?s",
                            $name
                        );
                        if ($q->num_rows > 0) {
                            while ($res = $q->fetch_assoc()) {
                                $type = mime_content_type(ROOT . $res['src']);
                                $size = filesize(ROOT . $res['src']);
                                $xml .= '
			<enclosure url="' . $this->getBaseurl() . $res['src'] . '" type="' . $type . '" length="' . $size . '"/>';
                            }
                        }
                    }
                }

                if ($row['parent_url'] == 'telestudiya') {
                    $xml .= '
			<enclosure url="https://youtu.be/' . $row['video_youtube'] . '" type="application/x-shockwave-flash" />';
                }

                if (!empty($row['images'])) {
                    foreach (explode('|', $row['images']) as $key => $img) {
                        $type = mime_content_type(ROOT . $img);
                        $size = filesize(ROOT . $img);
                        $xml .= '
			<enclosure url="' . $this->getBaseurl() . $img . '" type="' . $type . '" length="' . $size . '"/>';
                    }

                }
                $xml .= '
		</item>';
            }
            $xml .= '
	</channel>
</rss>';

            file_put_contents(ROOT . '/' . $rss, $xml);
        }
    }

    public function mail($email, $subject, $message, $files = false, $html = true)
    {
        try {
            $mail = new \core\Mail;
            $mail->from('no_reply@' . $_SERVER['SERVER_NAME'], \g::settings('company_name'));
            $mail->to($email);
            $mail->subject = $subject;
            $mail->body = $message;
            $mail->isHTML = $html;

            if ($files !== false)
                if (is_array($files))
                    foreach ($files as $file)
                        $mail->addFile($file);
                else
                    $mail->addFile($files);

            $mail->send();
            return true;
        } catch (\Exception $e) {
            return $e->getMessage();
        }
    }

    public function optimize_output_html($html)
    {
        $html = trim(preg_replace('/\s+/u', ' ', $html));
        $html = str_replace("> <", "><", $html);
        $html = preg_replace('/<!--(.*)-->/Uis', '', $html);
        return $html;
    }
}
