<?php
namespace globals;

class Template extends \core\response\Html
{
    private array $listCssJs = [];
    private array $listRightMenu = [];
    private $nav = null;

    public function getLocalLinks($module)
    {
        $list = \g::ml()->infoModules();

        foreach ($list as $item) {
            if ($item['name'] == $module) {
                if (array_key_exists('local', $item))
                    return $item['local'];
            }
        }
        return false;
    }

    public function getAdminPage($module, $template, $echo = true)
    {
        if (\g::stack()->get('route')['type'] !== 'admin')
            return false;
        $file = self::$_module_dir . ucfirst($module) . "/" . $template;

        if (file_exists($file . '.php')) {
            $file .= '.php';
        } else {
            if ($echo) {
                echo $echo;
                return;
            }
            return $file . '.php' . ' ==> Ошибка';
        }

        $LM = \g::module(ucfirst($module)); // Загрузим текущий класс модуля в локальную переменную для удобства.
        $listLocal = $this->getLocalLinks(ucfirst($module));
        // если в файле Module.php есть переменная local
        if ($listLocal !== false) {

            // Если есть локальные css файлы для подключения
            if(isset($listLocal['css'])){
                foreach ($listLocal['css'] as $link) {
                    $file_local = str_replace('.', '/', $link).'.css';
                    $this->linkTopCSS($file_local);
                }
            }

            // Если есть локальные js файлы для подключения
            if(isset($listLocal['js'])){
                foreach ($listLocal['js'] as $link) {
                    $file_local = str_replace('.', '/', $link).'.js';
                    $this->linkTopJS($file_local);
                }
            }

            // Если есть локальные js_bot файлы для подключения
            if(isset($listLocal['js_bot'])){
                foreach ($listLocal['js_bot'] as $link) {
                    $file_local = str_replace('.', '/', $link).'.js';
                    $this->linkBotJS($file_local.'.js');
                }
            }

            // Если есть локальные php файлы для подключения
            if(isset($listLocal['php'])){
                foreach ($listLocal['php'] as $link) {
                    $file_local = self::$_module_dir . ucfirst($module) . "/" . str_replace('.', '/', $link);

                    if (file_exists($file_local . '.php')) {
                        include_once $file_local . '.php';
                    }
                }
            }
        }

        $ml = \g::ml();
        ob_start();
        include $file;
        $content = ob_get_clean();

        if ($echo) {
            echo $content;
            return;
        }
        return $content;
    }

    public function chunk($template)
    {
        echo $this->getChunk($template);
    }

    public function getChunk($template)
    {
        $file = self::$_users_dir . 'chunks/' . $template;

        if (file_exists($file . '.php')) {
            $file .= '.php';
        } else {
            return false;
        }

        $ml = \g::ml();
        ob_start();
        include $file;
        $content = ob_get_clean();
        $event = new \core\event\events\ChunkRenderEvent($content, $file);
        $ml->fire($event);
        if ($event->isCancelled()) {
            return;
        }
        return $event->getContent();
    }

    public function getMenu()
    {
        if (!is_null($this->nav))
            return $this->nav;
        set_time_limit(0);
        $query = \g::db()->query("
			SELECT `id`, `url`, `name_abbr` , `name`, `nav_parent_url`, `prefix_url`, `sort_nav` AS `sort`, ('pages') AS `module`
			FROM `pages` 
			WHERE `is_nav` = 1 AND `active` = 1
			UNION
			SELECT `id`, `url`, `name_abbr`, `name`, `nav_parent_url`, ('catalog/category') AS prefix_url, `sort_nav` AS `sort`, ('catalog/categories') AS `module`
			FROM `catalog_categories`
			WHERE `is_nav` = 1 AND `active` = 1
			UNION			
			SELECT `id`, `url`, `name_abbr`, `name`, `nav_parent_url`, ('catalog/product') AS prefix_url, `sort_nav` AS `sort`, ('catalog/products') AS `module`
			FROM `catalog_products`
			WHERE `is_nav` = 1 AND `active` = 1
			ORDER BY `nav_parent_url`, `sort` ASC"
        );
        $nav = [];
        if ($query->num_rows > 0) {

            while ($row = $query->fetch_assoc()) {
				$row['name'] = stripslashes($row['name']);
                $row['menu_url'] = \f::generateUrl($row['url'], $row['prefix_url'], $row['module']);
                if (empty($row['nav_parent_url'])) {
                    if (!isset($nav[$row['url']])) {
                        $nav[$row['url']] = [];
                    }
                    $nav[$row['url']] = array_merge($nav[$row['url']], $row);
                } else {
                    $nav[$row['nav_parent_url']]['children'][] = $row;
                }
            }

            if (method_exists(\g::Multilingual(), 'getLangList') && \g::multilingual()->checkRender()) {
                $pages_ids = [];
                $categories_ids = [];
                $products_ids = [];

                foreach ($nav as $item) {
                    switch ($item['module']) {
                        case 'pages':
                            $pages_ids[] = $item['id'];
                            break;
                        case 'catalog/categories':
                            $categories_ids[] = $item['id'];
                            break;
                        case 'catalog/products':
                            $products_ids[] = $item['id'];
                            break;
                    }
                }
                $fields = ['name', 'name_abbr', 'title', 'keywords', 'description', 'anons', 'content', 'parent_id'];

                $pages_ids = \g::db()->getAll('
                    SELECT ' . implode(', ', $fields) . '
                    FROM pages_lang WHERE `parent_id` IN (?a) AND `lang` = ?s', $pages_ids, \g::stack()->get('lang'));

                if (count($categories_ids))
                    $categories_ids = \g::db()->getAll('
                    SELECT ' . implode(', ', $fields) . '
                    FROM catalog_categories_lang WHERE `parent_id` IN (?a) AND `lang` = ?s', $categories_ids, \g::stack()->get('lang'));

                if (count($products_ids))
                    $products_ids = \g::db()->getAll('
                    SELECT ' . implode(', ', $fields) . '
                    FROM catalog_products_lang WHERE `parent_id` IN (?a) AND `lang` = ?s', $products_ids, \g::stack()->get('lang'));

                foreach ($nav as &$item) {
                    $data = [];
                    switch ($item['module']) {
                        case 'pages':
                            $data = $pages_ids;
                            break;
                        case 'catalog/categories':
                            $data = $categories_ids;
                            break;
                        case 'catalog/products':
                            $data = $products_ids;
                            break;
                    }

                    foreach ($data as $newItem) {
                        if ($newItem['parent_id'] != $item['id'])
                            continue;

                        foreach ($fields as $field)
                        {
                            $item[$field] = $newItem[$field];
                        }
                    }
                }

            }


            return $nav;
        }
        return false;
    }

    public function pagination($linkFormat, $pageFormat, $ellipsis, $activePage, $totalPage, $context)
    {
        return \core\PaginationLinks::create(
            $activePage,
            $totalPage,
            $context,
            $linkFormat,
            $pageFormat,
            $ellipsis
        );
    }

    // helper fcn for linkCSS, linkJS.( linkers-fcn )
    private function getFilesFromModDir($nameDir,$from,$nameFile = false){
        $modName = \g::stack()->get('route')['modules'];

        if($nameFile && file_exists("./modules/$modName/$nameFile")){
            $this->listCssJs[$nameDir][$from][] = "/modules/$modName/$nameFile";
        } else {
            $files = scandir("./modules/$modName/$nameDir");
            if ($files){
                foreach($files as $file){ if ($file != "." && $file != ".."){
                    $this->listCssJs[$nameDir][$from][] = "/modules/$modName/$nameDir/$file";
                }}
            }

        }
    }
    // Вставит HTML код с подключаемыми файлами из модуля в head
    // $from - куда подключить "top" - head "bot" - в конец страници
    public function printLinkModCssJs($from = "top"){
        //подключим css files
        if(isset($this->listCssJs['css'][$from])) {
            foreach($this->listCssJs['css'][$from] as $file){
                echo '<link href="'.$file.'" rel="stylesheet" type="text/css"/>';
            }
        }
        // connect js files
        if(isset($this->listCssJs['js'][$from])) {
            foreach($this->listCssJs['js'][$from] as $file){
                echo ' <script src="'.$file.'"></script>';
            }
        }
    }
    // в head Подключит CSS файлы из модуля которые лежат в папке css, либо 1 файл если передать в переменную путь ( относително модуля ).
    public function linkTopCSS($nameFile = false){
        $this->getFilesFromModDir('css','top',$nameFile);
    }
    // в head Подключит JS файлы из модуля которые лежат в папке js, либо 1 файл если передать в переменную путь ( относително модуля ).
    public function linkTopJS($nameFile = false){
        $this->getFilesFromModDir('js','top',$nameFile);
    }
    // в самый низ страници Подключит CSS файлы из модуля которые лежат в папке css, либо 1 файл если передать в переменную путь ( относително модуля ).
    public function linkBotCSS($nameFile = false){
        $this->getFilesFromModDir('css','bot',$nameFile);
    }
    //  в самый низ страници Подключит JS файлы из модуля которые лежат в папке js, либо 1 файл если передать в переменную путь ( относително модуля ).
    public function linkBotJS($nameFile = false){
        $this->getFilesFromModDir('js','bot',$nameFile);
    }
}
