<?php
namespace globals;

class Validate
{
	private $errors = [];

	public function isErrors() {
		return empty($this->errors) ? false : true;
	}

	public function sendErrors() {
		return $this->errors;
	}

	public function int($value) {
		if (is_numeric($value)) {
			return (int) $value;
		}
		$this->errors[] = [
			'name' => 'Ошибка проверки значения',
			'message' => 'Вы передали на проверку не число'
		];
		return false;
	}

	public function float($value) {
		if (is_numeric($value)) {
			return (float) $value;
		}
		$this->errors[] = [
			'name' => 'Ошибка проверки значения',
			'message' => 'Вы передали на проверку не число'
		];
		return false;
	}

	public function string($value) {
		if (!empty($value) && is_string($value)) {
			return htmlspecialchars(addslashes($value));
		}else if(is_numeric($value) && $value == 0){
            return htmlspecialchars(addslashes($value));
        } else {
			return '';
		}
	}

	public function onlyLetters($value) {
		if (!empty($value) && is_string($value)) {
			return preg_replace('/[^ a-zйцукенгшщзхъфывапролджэячсмитьбюё\d]/ui', '', mb_strtolower($value));
		} else {
			return '';
		}
	}

	public function array($values) {
		if (is_array($values)) {
			$result = [];
			foreach ($values as $key => $value) {
				if (is_array($value)) {
					$result[$key] = $this->array($value);
				} else {
					if ($key === 'keywords') {
						$result[$key] = $this->onlyLetters($value);
					} else {
						$result[$key] = $this->string($value);
					}
				}
			}
			return $result;
		}
		$this->errors[] = [
			'name' => 'Не массив',
			'message' => 'Вы передали на проверку не массив'
		];
		return false;
	}

	public function name($value) {
		return $this->string($value);
	}

	public function text($value) {
		return $this->string($value);
	}

	public function fromDateToTime($date) {
		$time = $this->time($date);
		return $time ? $time : time();
	}

	public function time($value) {
		if (is_numeric($value)) {
			return $value;
		} else {
			$value = strtotime($value);
			return is_numeric($value) ? $value : false;
		}
	}

	public function html($value) {
		if (!empty($value)) {
			return htmlspecialchars(addslashes($value));
		}
		$this->errors[] = [
			'name' => 'Отсутствует значение',
			'message' => 'Значение для проверки не может быть пустым'
		];
		return false;
	}

	public function json($value, $isArray = true) {
		if (!empty($value)) {
			$array = json_decode($value, true);
			if ($isArray) {
				return $this->array($array);
			} else {
				return json_encode($this->array($array) ?? []);
			}
		}
		$this->errors[] = [
			'name' => 'Отсутствует значение',
			'message' => 'Значение для проверки не может быть пустым'
		];
		return false;		
	}
}